# Generated by Django 2.2.4 on 2019-08-10 14:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('minke', '0004_auto_20190703_1319'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='minkesession',
            name='task_id',
        ),
        migrations.AddField(
            model_name='minkesession',
            name='pid',
            field=models.IntegerField(blank=True, help_text='Process-ID of the celery-task that run the session.', null=True, verbose_name='PID'),
        ),
    ]
