{% extends 'admin/change_form.html' %}
{% load i18n minke admin_urls %}

{% block object-tools-items %}
{% if object_id %}
    {% url 'admin:minke_minkesession_changelist' as session_log_url %}
    <li><a href="{{ session_log_url }}?minkeobj_type__id__exact={{ content_type_id }}&minkeobj_id={{ object_id }}">
        {% trans "session-log" %}
    </a></li>
{% endif %}
{{ block.super }}
{% endblock %}
