/*
CSS for changelist of minke-objects.
It is also used by the session's changelist itself.
 */

/* session-switch (session's changlist only) */
div.session_switch {
    margin-top: 10px;
}

/* session-summary (minke-objects' changlists only) */
div.session_select span.session-summary span {
    margin-left: 8px;
    font-size: 13px;
}

/* message-toggle (minke-objects' changlists only) */
div.session_select a.message-toggle,
tr.session a.message-toggle {
    font-size: 13px;
    float: right;
    cursor: pointer;
    color: #999;
}

/* session-info-row */
table.sessions tr.session div.session_proc_info {
    margin-bottom: 10px;
}

table.sessions tr.session td {
    color: #666;
}

table.sessions tr.session *[title] {
    cursor: help;
}

table.sessions tr.minkeobj.session-info > * {
    border: 0;
}

/* message-list-css */
table.sessions tr.session[data-msg-count="0"] a.message-toggle {
    display: none;
}

table.sessions ul.messagelist.hide {
    display: none;
}

table.sessions ul.messagelist pre {
    margin: 0;
    padding: 0;
}

table.sessions ul.messagelist li {
    padding-left: 45px;
    border: 1px solid #eee;
    background-position: 12px 12px;
    overflow: auto;
    position: relative;
    left: -8px;
    /* max-height: 200px; */
}

table.sessions ul.messagelist li:last-child {
    margin: 0;
}

/* background-color for sessions and messages */
table.sessions tr.row1.success,
table.sessions tr.row1 li.info {
    background-color: #e8ffe8;
}
table.sessions tr.row2.success,
table.sessions tr.row2 li.info {
    background-color: #dfd;
}
table.sessions tr.row1.warning,
table.sessions tr.row1 li.warning {
    background-color: #ffffe6;
}
table.sessions tr.row2.warning,
table.sessions tr.row2 li.warning {
    background-color: #ffc;
}
table.sessions tr.row1.error,
table.sessions tr.row1 li.error {
    background-color: #fff6f6;
}
table.sessions tr.row2.error,
table.sessions tr.row2 li.error {
    background-color: #ffefef;
}

/* Session-stopper css (minke-objects' changlists only) */
table.sessions div.session_stopper::after {
    content: '✕';
    font-size: 18px;
    font-weight: normal;
    cursor: pointer;
}

table.sessions tr.minkeobj.running div.session_stopper,
table.sessions tr.minkeobj.stopping div.session_stopper,
table.sessions.running th div.session_stopper {
    color: red;
}

table.sessions tr.minkeobj.stopping div.session_stopper::after,
table.sessions.stopping th:first-child div.session_stopper::after {
    font-weight: bold;
}

table.sessions div.session_stopper,
table.sessions tr.minkeobj.initialized td:first-child input,
table.sessions tr.minkeobj.running td:first-child input,
table.sessions tr.minkeobj.stopping td:first-child input,
table.sessions.running th:first-child div.text {
    display: none;
}

table.sessions tr.minkeobj.initialized div.session_stopper,
table.sessions tr.minkeobj.running div.session_stopper,
table.sessions tr.minkeobj.stopping div.session_stopper,
table.sessions.running th:first-child div.session_stopper {
    display: block;
}
